%% SSE METAMODELING: Approximation of complex function
%
% This example showcases how SSE approximates a function with
% non-homogeneous complexity, i.e., a function with complexity that depends
% strongly on the input. It shows an approach with a static experimental
% design and compares the approximation to sparse PCE.

%% 1 - INITIALIZE UQLAB
clearvars
uqlab
%% 2 - FORWARD MODEL
%% case 1
% %%
% FILELOCATION = fullfile(uq_rootPath, 'Examples', 'SimpleDataSets',...
%     'supportbeam');
% load(fullfile(FILELOCATION,'Supportbeam_DOE40.mat'), 'X', 'Y');
% load(fullfile(FILELOCATION,'Supportbeam_VAL30.mat'), 'Xval', 'Yval');
% %% 3 - PRIOR DISTRIBUTION OF THE MODEL PARAMETERS
% Input.Marginals(1).Name = 'E';               % Young's modulus
% Input.Marginals(1).Type = 'Lognormal';  
% Input.Marginals(1).Moments = [30e9 4.5e9];   % (N/m^2)
% myInput = uq_createInput(Input);

%% case 2
%%
FILELOCATION = fullfile(uq_rootPath, 'Examples', 'SimpleDataSets',...
    'plate_shell');
load(fullfile(FILELOCATION,'plate_shell_DOE25.mat'), 'X', 'Y');
load(fullfile(FILELOCATION,'plate_shell_VAL20'), 'Xval', 'Yval');

InputOpts.Marginals(1).Name = 'E'; %Concrete modulus of elasticity
InputOpts.Marginals(1).Type = 'Lognormal';
InputOpts.Marginals(1).Moments = [7.1e8 7.1e7]; % (Pa)

InputOpts.Marginals(2).Name = 'D'; %Concrete modulus of elasticity
InputOpts.Marginals(2).Type = 'Lognormal';
InputOpts.Marginals(2).Moments = [2700 270]; % (Pa)

myInput = uq_createInput(InputOpts);
%% 4 - STOCHASTIC SPECTRAL EMBEDDING (SSE) METAMODEL
%
% This section showcases a simple way to calculate a stochastic spectral 
% embedding (SSE) metamodel.

%%
% Select SSE as the metamodeling tool in UQLab:
SSEOpts.Type = 'Metamodel';
SSEOpts.MetaType = 'SSE';

SSEOpts.ExpDesign.X = X;
SSEOpts.ExpDesign.Y = Y;


%% 4.1 - Residual expansion technique

SSEOpts.ExpOptions.Type = 'Metamodel';
SSEOpts.ExpOptions.MetaType = 'PCE';
SSEOpts.ExpOptions.Degree = 0:5;
MetaOpts.Method = 'LARS' ;
MetaOpts.TruncOptions.qNorm = 0.75;
%%
% Specify the sample size required in a subdomain to create an expansion:
SSEOpts.Refine.NExp = 3;

%% 
% Computing the first two output moments requires computing the flattened 
% SSE representation. This needs some additional computational effort and, 
% therefore, needs to be enabled explicitly:
% SSEOpts.PostProcessing.outputMoments = true;

%%
% Create the SSE metamodel:
mySSE = uq_createModel(SSEOpts);

%%
% Print a report on the constructed surrogate:
uq_print(mySSE)

%%
% Create a visual representation of the SSE metamodel:
myFigures = uq_display(mySSE,1);

%% 5 - POLYNOMIAL CHAOS EXPANSION (PCE) METAMODEL
%
% We compare the SSE performance to PCE on the same experimental design.

%%
% Select degree-adaptive PCE as the metamodeling tool in UQLab:
PCEOpts.Type = 'metamodel';
PCEOpts.MetaType = 'PCE';
PCEOpts.Degree = 0:5;
MetaOpts.Method = 'LARS' ;
MetaOpts.TruncOptions.qNorm = 0.75;
%%
% Choose the same experimental design that was used to construct |mySSE|:
PCEOpts.ExpDesign.X = mySSE.ExpDesign.X;
PCEOpts.ExpDesign.Y = mySSE.ExpDesign.Y;

%%
% Create the PCE metamodel:
myPCE = uq_createModel(PCEOpts);

%% 6 - VALIDATION OF THE METAMODELS

%% 6.1 - Generation of a validation set

MetaOpts.ValidationSet.X = Xval;
MetaOpts.ValidationSet.Y = Yval;
%%
% Evaluate the full model response at the validation sample points:
%Yval = uq_evalModel(myModel,Xval);

%%
% Evaluate the corresponding responses for each of the metamodels created 
% before:
YSSE = uq_evalModel(mySSE,Xval);
YPCE = uq_evalModel(myPCE,Xval);

%%
% Store the metamodels responses in |YContainer|:
YContainer = {YPCE, YSSE};

%% 6.2 - Computation of the validation error
%
% Compute the validation error for the PCE and SSE metamodels.
mySSEs = {myPCE, mySSE};
methodLabels = {'PCE', 'SSE'};

fprintf('Validation error:\n')
fprintf('%20s | Rel. error | ED size\n', 'Method')
fprintf('--------------------------------------------\n')
for ii = 1:length(YContainer)
    normEmpErr = mean((Yval - YContainer{ii}).^2)/var(Yval);
    fprintf('%20s | %10.2e | %7d\n', methodLabels{ii}, normEmpErr, mySSEs{ii}.ExpDesign.NSamples);
end

%% 6.3 - Comparison of the results
%
% % To visually assess the performance of each metamodel, we add a plot of 
% % the PCE response to the SSE plot:
plotAx = myFigures{1}.Children(2);
plotPCEX = plotAx.Children(2).XData.';
plotPCEY = uq_evalModel(myPCE, plotPCEX);
uq_plot(plotAx, plotPCEX, plotPCEY,'Color',[1 1 1]*0.7,'DisplayName','$\mathcal{M}_{\mathrm{PCE}}$')

%%
%
% Additionally, we create scatter plots of the metamodel vs. the true 
% response on the validation sets:
uq_figure('Name', 'Metamodels vs. true model')

for ii = 1:length(YContainer)
    subplot(1,length(YContainer),ii)
    uq_plot(Yval, YContainer{ii}, '+')
    hold on
    uq_plot([min(Yval) max(Yval)], [min(Yval) max(Yval)], 'k')
    hold off
    axis equal 
    axis([min(Yval) max(Yval) min(Yval) max(Yval)]) 
    
    title(methodLabels{ii})
    xlabel('$\mathrm{Y_{true}}$')
    if strcmpi(methodLabels{ii},'PCE')
        ylabel('$\mathrm{Y_{PCE}}$')
    else
        ylabel('$\mathrm{Y_{SSE}}$')
    end
end

%%
% *References*
%
% * Marelli, S., Wagner, P.-R., Lataniotis, C., Sudret, B., Stochastic 
%   spectral embedding, International Journal for Uncertainty 
%   Quantification, 11(2):25?47(2021). 
%   DOI: <https://doi.org/10.1615/Int.J.UncertaintyQuantification.2020034395 10.1615/Int.J.UncertaintyQuantification.2020034395>